# This Python file uses the following encoding: utf-8
"""autogenerated by genpy from intera_core_msgs/InteractionControlCommand.msg. Do not edit."""
import sys
python3 = True if sys.hexversion > 0x03000000 else False
import genpy
import struct

import geometry_msgs.msg
import std_msgs.msg

class InteractionControlCommand(genpy.Message):
  _md5sum = "762b5d197d5d786f83741af7fe6261a6"
  _type = "intera_core_msgs/InteractionControlCommand"
  _has_header = True #flag to mark the presence of a Header object
  _full_text = """# Message sets the interaction (impedance/force) control on or off
# It also contains desired cartesian stiffness K, damping D, and force values

Header header
bool      interaction_control_active

## Cartesian Impedance Control Parameters
# Stiffness units are (N/m) for first 3 and (Nm/rad) for second 3 values
float64[] K_impedance
# Force certain directions to have maximum possible impedance for a given pose
bool[] max_impedance
# Damping units are (Ns/m) for first 3 and (Nms/rad) for the second 3 values
float64[] D_impedance
# Joint Nullspace stiffness units are in (Nm/rad) (length == number of joints)
float64[] K_nullspace

## Parameters for force control or impedance control with force limit
# If in force mode, this is the vector of desired forces/torques
# to be regulated in (N) and (Nm)
# If in impedance with force limit mode, this vector specifies the
# magnitude of forces/torques (N and Nm) that the command will not exceed.
float64[] force_command

## Desired frame
geometry_msgs/Pose interaction_frame
string endpoint_name
# True if impedance and force commands are defined in endpoint frame
bool in_endpoint_frame

# Set to true to disable damping during force control. Damping is used
# to slow down robot motion during force control in free space.
# Option included for SDK users to disable damping in force control
bool disable_damping_in_force_control

# Set to true to disable reference resetting. Reference resetting is
# used when interaction parameters change, in order to avoid jumps/jerks.
# Option included for SDK users to disable reference resetting if the
# intention is to change interaction parameters.
bool disable_reference_resetting

## Mode Selection Parameters
# The possible interaction control modes are:
# Impedance mode: implements desired endpoint stiffness and damping.
uint8 IMPEDANCE_MODE=1
# Force mode: applies force/torque in the specified dimensions.
uint8 FORCE_MODE=2
# Impedance with force limit: impedance control while ensuring the commanded
# forces/torques do not exceed force_command.
uint8 IMPEDANCE_WITH_FORCE_LIMIT_MODE=3
# Force with motion bounds: force control while ensuring the current
# pose/velocities do not exceed forceMotionThreshold (currenetly defined in yaml)
uint8 FORCE_WITH_MOTION_LIMIT_MODE=4

# Specifies the interaction control mode for each Cartesian dimension (6)
uint8[] interaction_control_mode

# All 6 values in force and impedance parameter vectors have to be filled,
# If a control mode is not used in a Cartesian dimension,
# the corresponding parameters will be ignored.

## Parameters for Constrained Zero-G Behaviors
# Allow for arbitrary rotational displacements from the current orientation
# for constrained zero-G. Setting 'rotations_for_constrained_zeroG = True'
# will disable the rotational stiffness field which limits rotational
# displacements to +/- 82.5 degree.
# NOTE: it will be only enabled for a stationary reference orientation
bool rotations_for_constrained_zeroG

================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.sec: seconds (stamp_secs) since epoch (in Python the variable is called 'secs')
# * stamp.nsec: nanoseconds since stamp_secs (in Python the variable is called 'nsecs')
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
# 0: no frame
# 1: global frame
string frame_id

================================================================================
MSG: geometry_msgs/Pose
# A representation of pose in free space, composed of position and orientation. 
Point position
Quaternion orientation

================================================================================
MSG: geometry_msgs/Point
# This contains the position of a point in free space
float64 x
float64 y
float64 z

================================================================================
MSG: geometry_msgs/Quaternion
# This represents an orientation in free space in quaternion form.

float64 x
float64 y
float64 z
float64 w
"""
  # Pseudo-constants
  IMPEDANCE_MODE = 1
  FORCE_MODE = 2
  IMPEDANCE_WITH_FORCE_LIMIT_MODE = 3
  FORCE_WITH_MOTION_LIMIT_MODE = 4

  __slots__ = ['header','interaction_control_active','K_impedance','max_impedance','D_impedance','K_nullspace','force_command','interaction_frame','endpoint_name','in_endpoint_frame','disable_damping_in_force_control','disable_reference_resetting','interaction_control_mode','rotations_for_constrained_zeroG']
  _slot_types = ['std_msgs/Header','bool','float64[]','bool[]','float64[]','float64[]','float64[]','geometry_msgs/Pose','string','bool','bool','bool','uint8[]','bool']

  def __init__(self, *args, **kwds):
    """
    Constructor. Any message fields that are implicitly/explicitly
    set to None will be assigned a default value. The recommend
    use is keyword arguments as this is more robust to future message
    changes.  You cannot mix in-order arguments and keyword arguments.

    The available fields are:
       header,interaction_control_active,K_impedance,max_impedance,D_impedance,K_nullspace,force_command,interaction_frame,endpoint_name,in_endpoint_frame,disable_damping_in_force_control,disable_reference_resetting,interaction_control_mode,rotations_for_constrained_zeroG

    :param args: complete set of field values, in .msg order
    :param kwds: use keyword arguments corresponding to message field names
    to set specific fields.
    """
    if args or kwds:
      super(InteractionControlCommand, self).__init__(*args, **kwds)
      #message fields cannot be None, assign default values for those that are
      if self.header is None:
        self.header = std_msgs.msg.Header()
      if self.interaction_control_active is None:
        self.interaction_control_active = False
      if self.K_impedance is None:
        self.K_impedance = []
      if self.max_impedance is None:
        self.max_impedance = []
      if self.D_impedance is None:
        self.D_impedance = []
      if self.K_nullspace is None:
        self.K_nullspace = []
      if self.force_command is None:
        self.force_command = []
      if self.interaction_frame is None:
        self.interaction_frame = geometry_msgs.msg.Pose()
      if self.endpoint_name is None:
        self.endpoint_name = ''
      if self.in_endpoint_frame is None:
        self.in_endpoint_frame = False
      if self.disable_damping_in_force_control is None:
        self.disable_damping_in_force_control = False
      if self.disable_reference_resetting is None:
        self.disable_reference_resetting = False
      if self.interaction_control_mode is None:
        self.interaction_control_mode = b''
      if self.rotations_for_constrained_zeroG is None:
        self.rotations_for_constrained_zeroG = False
    else:
      self.header = std_msgs.msg.Header()
      self.interaction_control_active = False
      self.K_impedance = []
      self.max_impedance = []
      self.D_impedance = []
      self.K_nullspace = []
      self.force_command = []
      self.interaction_frame = geometry_msgs.msg.Pose()
      self.endpoint_name = ''
      self.in_endpoint_frame = False
      self.disable_damping_in_force_control = False
      self.disable_reference_resetting = False
      self.interaction_control_mode = b''
      self.rotations_for_constrained_zeroG = False

  def _get_types(self):
    """
    internal API method
    """
    return self._slot_types

  def serialize(self, buff):
    """
    serialize message into buffer
    :param buff: buffer, ``StringIO``
    """
    try:
      _x = self
      buff.write(_get_struct_3I().pack(_x.header.seq, _x.header.stamp.secs, _x.header.stamp.nsecs))
      _x = self.header.frame_id
      length = len(_x)
      if python3 or type(_x) == unicode:
        _x = _x.encode('utf-8')
        length = len(_x)
      buff.write(struct.pack('<I%ss'%length, length, _x))
      buff.write(_get_struct_B().pack(self.interaction_control_active))
      length = len(self.K_impedance)
      buff.write(_struct_I.pack(length))
      pattern = '<%sd'%length
      buff.write(struct.pack(pattern, *self.K_impedance))
      length = len(self.max_impedance)
      buff.write(_struct_I.pack(length))
      pattern = '<%sB'%length
      buff.write(struct.pack(pattern, *self.max_impedance))
      length = len(self.D_impedance)
      buff.write(_struct_I.pack(length))
      pattern = '<%sd'%length
      buff.write(struct.pack(pattern, *self.D_impedance))
      length = len(self.K_nullspace)
      buff.write(_struct_I.pack(length))
      pattern = '<%sd'%length
      buff.write(struct.pack(pattern, *self.K_nullspace))
      length = len(self.force_command)
      buff.write(_struct_I.pack(length))
      pattern = '<%sd'%length
      buff.write(struct.pack(pattern, *self.force_command))
      _x = self
      buff.write(_get_struct_7d().pack(_x.interaction_frame.position.x, _x.interaction_frame.position.y, _x.interaction_frame.position.z, _x.interaction_frame.orientation.x, _x.interaction_frame.orientation.y, _x.interaction_frame.orientation.z, _x.interaction_frame.orientation.w))
      _x = self.endpoint_name
      length = len(_x)
      if python3 or type(_x) == unicode:
        _x = _x.encode('utf-8')
        length = len(_x)
      buff.write(struct.pack('<I%ss'%length, length, _x))
      _x = self
      buff.write(_get_struct_3B().pack(_x.in_endpoint_frame, _x.disable_damping_in_force_control, _x.disable_reference_resetting))
      _x = self.interaction_control_mode
      length = len(_x)
      # - if encoded as a list instead, serialize as bytes instead of string
      if type(_x) in [list, tuple]:
        buff.write(struct.pack('<I%sB'%length, length, *_x))
      else:
        buff.write(struct.pack('<I%ss'%length, length, _x))
      buff.write(_get_struct_B().pack(self.rotations_for_constrained_zeroG))
    except struct.error as se: self._check_types(struct.error("%s: '%s' when writing '%s'" % (type(se), str(se), str(locals().get('_x', self)))))
    except TypeError as te: self._check_types(ValueError("%s: '%s' when writing '%s'" % (type(te), str(te), str(locals().get('_x', self)))))

  def deserialize(self, str):
    """
    unpack serialized message in str into this message instance
    :param str: byte array of serialized message, ``str``
    """
    try:
      if self.header is None:
        self.header = std_msgs.msg.Header()
      if self.interaction_frame is None:
        self.interaction_frame = geometry_msgs.msg.Pose()
      end = 0
      _x = self
      start = end
      end += 12
      (_x.header.seq, _x.header.stamp.secs, _x.header.stamp.nsecs,) = _get_struct_3I().unpack(str[start:end])
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      start = end
      end += length
      if python3:
        self.header.frame_id = str[start:end].decode('utf-8')
      else:
        self.header.frame_id = str[start:end]
      start = end
      end += 1
      (self.interaction_control_active,) = _get_struct_B().unpack(str[start:end])
      self.interaction_control_active = bool(self.interaction_control_active)
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      pattern = '<%sd'%length
      start = end
      end += struct.calcsize(pattern)
      self.K_impedance = struct.unpack(pattern, str[start:end])
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      pattern = '<%sB'%length
      start = end
      end += struct.calcsize(pattern)
      self.max_impedance = struct.unpack(pattern, str[start:end])
      self.max_impedance = map(bool, self.max_impedance)
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      pattern = '<%sd'%length
      start = end
      end += struct.calcsize(pattern)
      self.D_impedance = struct.unpack(pattern, str[start:end])
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      pattern = '<%sd'%length
      start = end
      end += struct.calcsize(pattern)
      self.K_nullspace = struct.unpack(pattern, str[start:end])
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      pattern = '<%sd'%length
      start = end
      end += struct.calcsize(pattern)
      self.force_command = struct.unpack(pattern, str[start:end])
      _x = self
      start = end
      end += 56
      (_x.interaction_frame.position.x, _x.interaction_frame.position.y, _x.interaction_frame.position.z, _x.interaction_frame.orientation.x, _x.interaction_frame.orientation.y, _x.interaction_frame.orientation.z, _x.interaction_frame.orientation.w,) = _get_struct_7d().unpack(str[start:end])
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      start = end
      end += length
      if python3:
        self.endpoint_name = str[start:end].decode('utf-8')
      else:
        self.endpoint_name = str[start:end]
      _x = self
      start = end
      end += 3
      (_x.in_endpoint_frame, _x.disable_damping_in_force_control, _x.disable_reference_resetting,) = _get_struct_3B().unpack(str[start:end])
      self.in_endpoint_frame = bool(self.in_endpoint_frame)
      self.disable_damping_in_force_control = bool(self.disable_damping_in_force_control)
      self.disable_reference_resetting = bool(self.disable_reference_resetting)
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      start = end
      end += length
      self.interaction_control_mode = str[start:end]
      start = end
      end += 1
      (self.rotations_for_constrained_zeroG,) = _get_struct_B().unpack(str[start:end])
      self.rotations_for_constrained_zeroG = bool(self.rotations_for_constrained_zeroG)
      return self
    except struct.error as e:
      raise genpy.DeserializationError(e) #most likely buffer underfill


  def serialize_numpy(self, buff, numpy):
    """
    serialize message with numpy array types into buffer
    :param buff: buffer, ``StringIO``
    :param numpy: numpy python module
    """
    try:
      _x = self
      buff.write(_get_struct_3I().pack(_x.header.seq, _x.header.stamp.secs, _x.header.stamp.nsecs))
      _x = self.header.frame_id
      length = len(_x)
      if python3 or type(_x) == unicode:
        _x = _x.encode('utf-8')
        length = len(_x)
      buff.write(struct.pack('<I%ss'%length, length, _x))
      buff.write(_get_struct_B().pack(self.interaction_control_active))
      length = len(self.K_impedance)
      buff.write(_struct_I.pack(length))
      pattern = '<%sd'%length
      buff.write(self.K_impedance.tostring())
      length = len(self.max_impedance)
      buff.write(_struct_I.pack(length))
      pattern = '<%sB'%length
      buff.write(self.max_impedance.tostring())
      length = len(self.D_impedance)
      buff.write(_struct_I.pack(length))
      pattern = '<%sd'%length
      buff.write(self.D_impedance.tostring())
      length = len(self.K_nullspace)
      buff.write(_struct_I.pack(length))
      pattern = '<%sd'%length
      buff.write(self.K_nullspace.tostring())
      length = len(self.force_command)
      buff.write(_struct_I.pack(length))
      pattern = '<%sd'%length
      buff.write(self.force_command.tostring())
      _x = self
      buff.write(_get_struct_7d().pack(_x.interaction_frame.position.x, _x.interaction_frame.position.y, _x.interaction_frame.position.z, _x.interaction_frame.orientation.x, _x.interaction_frame.orientation.y, _x.interaction_frame.orientation.z, _x.interaction_frame.orientation.w))
      _x = self.endpoint_name
      length = len(_x)
      if python3 or type(_x) == unicode:
        _x = _x.encode('utf-8')
        length = len(_x)
      buff.write(struct.pack('<I%ss'%length, length, _x))
      _x = self
      buff.write(_get_struct_3B().pack(_x.in_endpoint_frame, _x.disable_damping_in_force_control, _x.disable_reference_resetting))
      _x = self.interaction_control_mode
      length = len(_x)
      # - if encoded as a list instead, serialize as bytes instead of string
      if type(_x) in [list, tuple]:
        buff.write(struct.pack('<I%sB'%length, length, *_x))
      else:
        buff.write(struct.pack('<I%ss'%length, length, _x))
      buff.write(_get_struct_B().pack(self.rotations_for_constrained_zeroG))
    except struct.error as se: self._check_types(struct.error("%s: '%s' when writing '%s'" % (type(se), str(se), str(locals().get('_x', self)))))
    except TypeError as te: self._check_types(ValueError("%s: '%s' when writing '%s'" % (type(te), str(te), str(locals().get('_x', self)))))

  def deserialize_numpy(self, str, numpy):
    """
    unpack serialized message in str into this message instance using numpy for array types
    :param str: byte array of serialized message, ``str``
    :param numpy: numpy python module
    """
    try:
      if self.header is None:
        self.header = std_msgs.msg.Header()
      if self.interaction_frame is None:
        self.interaction_frame = geometry_msgs.msg.Pose()
      end = 0
      _x = self
      start = end
      end += 12
      (_x.header.seq, _x.header.stamp.secs, _x.header.stamp.nsecs,) = _get_struct_3I().unpack(str[start:end])
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      start = end
      end += length
      if python3:
        self.header.frame_id = str[start:end].decode('utf-8')
      else:
        self.header.frame_id = str[start:end]
      start = end
      end += 1
      (self.interaction_control_active,) = _get_struct_B().unpack(str[start:end])
      self.interaction_control_active = bool(self.interaction_control_active)
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      pattern = '<%sd'%length
      start = end
      end += struct.calcsize(pattern)
      self.K_impedance = numpy.frombuffer(str[start:end], dtype=numpy.float64, count=length)
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      pattern = '<%sB'%length
      start = end
      end += struct.calcsize(pattern)
      self.max_impedance = numpy.frombuffer(str[start:end], dtype=numpy.bool, count=length)
      self.max_impedance = map(bool, self.max_impedance)
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      pattern = '<%sd'%length
      start = end
      end += struct.calcsize(pattern)
      self.D_impedance = numpy.frombuffer(str[start:end], dtype=numpy.float64, count=length)
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      pattern = '<%sd'%length
      start = end
      end += struct.calcsize(pattern)
      self.K_nullspace = numpy.frombuffer(str[start:end], dtype=numpy.float64, count=length)
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      pattern = '<%sd'%length
      start = end
      end += struct.calcsize(pattern)
      self.force_command = numpy.frombuffer(str[start:end], dtype=numpy.float64, count=length)
      _x = self
      start = end
      end += 56
      (_x.interaction_frame.position.x, _x.interaction_frame.position.y, _x.interaction_frame.position.z, _x.interaction_frame.orientation.x, _x.interaction_frame.orientation.y, _x.interaction_frame.orientation.z, _x.interaction_frame.orientation.w,) = _get_struct_7d().unpack(str[start:end])
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      start = end
      end += length
      if python3:
        self.endpoint_name = str[start:end].decode('utf-8')
      else:
        self.endpoint_name = str[start:end]
      _x = self
      start = end
      end += 3
      (_x.in_endpoint_frame, _x.disable_damping_in_force_control, _x.disable_reference_resetting,) = _get_struct_3B().unpack(str[start:end])
      self.in_endpoint_frame = bool(self.in_endpoint_frame)
      self.disable_damping_in_force_control = bool(self.disable_damping_in_force_control)
      self.disable_reference_resetting = bool(self.disable_reference_resetting)
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      start = end
      end += length
      self.interaction_control_mode = str[start:end]
      start = end
      end += 1
      (self.rotations_for_constrained_zeroG,) = _get_struct_B().unpack(str[start:end])
      self.rotations_for_constrained_zeroG = bool(self.rotations_for_constrained_zeroG)
      return self
    except struct.error as e:
      raise genpy.DeserializationError(e) #most likely buffer underfill

_struct_I = genpy.struct_I
def _get_struct_I():
    global _struct_I
    return _struct_I
_struct_3B = None
def _get_struct_3B():
    global _struct_3B
    if _struct_3B is None:
        _struct_3B = struct.Struct("<3B")
    return _struct_3B
_struct_3I = None
def _get_struct_3I():
    global _struct_3I
    if _struct_3I is None:
        _struct_3I = struct.Struct("<3I")
    return _struct_3I
_struct_B = None
def _get_struct_B():
    global _struct_B
    if _struct_B is None:
        _struct_B = struct.Struct("<B")
    return _struct_B
_struct_7d = None
def _get_struct_7d():
    global _struct_7d
    if _struct_7d is None:
        _struct_7d = struct.Struct("<7d")
    return _struct_7d
